# Still to do:
#   - subclass with StringIO like unicode compatiblity
#   - check speed
"""
This clas provide a file like object to access a string or any object wich
define the buffer interface method.
The wrapped object must also define a .resize(n) method if the emulated file
need to be able to change it's size.
"""

try:
    from errno import EINVAL
except ImportError:
    EINVAL = 22

from mem import chunk
from mem import tools
from mem import memslice


class StrIO(object):
    __index  = 0
    __closed = False

    def __init__(self, data=None):
        if data is None:
            data = chunk.chunk()
        self.data = data

    def __check(self):
        if self.__closed:
            raise ValueError, "I/O opperation on closed file"

    def read(self, n=-1):
        self.__check()
        start = self.__index
        if n<0:
            stop = tools.memlen(self.data)
        else:
            stop = min(start+n, tools.memlen(self.data))
        self.__index = stop
        return str(memslice.memslice(self.data, start, stop-start))

    def readline(self, length=None):
        self.__check()
        start = self.__index
        try:
            stop = tools.index(self.data, '\n', start)+1
        except ValueError:
            stop =  tools.memlen(self.data)
        if length is not None:
            stop = min(stop, start+length)
        self.__index = stop
        return buffer(self.data)[start:stop]

    def readlines(self, sizehint=0):
        total = 0
        lines = []
        while True:
            ln = self.readline()
            if not ln:
                break
            lines.append(ln)
            total += len(ln)
            if 0 < sizehint <= total:
                break
        return lines

    def isatty(self):
        self.__check()
        return False

    def seek(self, pos, mode=0):
        self.__check()
        if mode == 1:
            pos += self.__index
        elif mode == 2:
            pos += len(self)
        self.__index = max(0, pos)

    def tell(self):
        self.__check()
        return self.__index()

    def close(self):
        """does not actually free the memory...."""
        self.__closed = True

    # Iterator protocol
    def __iter__(self):
        return self
    def next(self):
        if self.__closed:
            raise StopIteration
        r = self.readline()
        if not r:
            raise StopIteration
        return r

    def getvalue(self):
        return str(self.data)

    # -------------------------------------------------
    # Part wich need a mutable data object
    # -------------------------------------------------
    def write(self, s):
        self.__check()
        index = self.__index
        limit = index + tools.memlen(s)
        try:
            tools.memcopy(self.data, s, index)
        except IndexError:
            self.data.resize(limit)
            tools.memcopy(self.data, s, index)
        self.__index = limit

    def writelines(self, lines):
        self.write(''.join(lines))

    def truncate(self, size=None):
        self.__check()
        if size is None:
            size = self.__index
        elif size < 0:
            raise IOError(EINVAL, "Negative size not allowed")
        self.__index = min(size, self.__index)
        self.data.resize(size)

def StringIO(s=""):
    return StrIO(chunk.chunk(s))
