"""
A replacement for array.array index method.

This should be more efficient for long arrays since the
original index method create a PyObject for each index
to check for. This mean also, that this functions are not
stricly equivalent.

This module is also a little more powerfull since it allow
to search backward and to provide start and stop limits.

Note that you can find mostly equivalent code into the
StrArray module.

"""


import array
import mem.tools as tools

def a_index(arr, value, start=0, stop=None):
    pattern = array.array(arr.typecode, (value,))
    step = arr.itemsize
    if stop is None:
        stop = tools.memlen(arr)
    else:
        stop *= step
    start *= step
    return tools.index(arr, pattern, start, stop, step)/step

def a_rindex(arr, value, start=0, stop=None):
    pattern = array.array(arr.typecode, (value,))
    step = arr.itemsize
    if stop is None:
        stop = tools.memlen(arr)
    else:
        stop *= step
    start *= step
    return tools.rindex(arr, pattern, start, stop, step)/step

if __name__ == '__main__':
    x = array.array('i', range(100))
    x.append(67)
    print a_index(x, 67)
    print a_index(x, 67,70)
    print a_rindex(x, 67)
    print a_rindex(x, 67,10,-30)

    # In some situations, you can't replace the index method
    # by the previous functions
    # print x.index(complex(10))
    # print a_index(x, complex(10))

