# Check that we can define a pure python array class.

import unittest
from mem.StrArray import Array, StrArray
from mem.memslice import memslice
from mem.chunk import chunk

class StrArrayTester(unittest.TestCase):
    def test_create(self):
        self.assertEqual(list(Array("i", (1,2,3))), [1 ,2, 3])
        self.assertRaises(Exception, Array, "i", (1,"x"))

    def test_getsetitem(self):
        x = Array("i", (1, 2, 3))
        self.assertEqual(x[0], 1)
        x[0] = 2
        self.assertEqual(x[0], 2)
        self.assertEqual(x[2], 3)
        del x[0]
        self.assertEqual(x, Array("i", (2,3)))
        self.assertRaises(IndexError, x.__getitem__, 10)
        self.assertRaises(IndexError, x.__setitem__, 10, 3)
        self.assertRaises(IndexError, x.__delitem__, 10)

    def test_getsetslice(self):
        x = Array("i", (1, 2, 3))
        self.assertEqual(x[:], x)
        self.assertEqual(x[0:0], Array("i"))
        self.assertEqual(x[1:], Array("i",[2,3]))
        self.assertEqual(x[:-1], Array("i", [1, 2]))

        # slicing by copy
        buffer = chunk("hello")
        x = StrArray("c", buffer)
        y = x[:]
        x[0] = "H"
        self.assertEqual(y, "hello")
        # slicing by view
        buffer = memslice(chunk("hello"))
        x = StrArray("c", buffer)
        y = x[:]
        x[0] = "H"
        self.assertEqual(y, "Hello")
        
        x = Array("i", (1, 2, 3))
        x[:] = []
        self.assertEqual(x, Array("i"))
        x = Array("i", (1, 2, 3))
        x[2:] = [10, 11, 12]
        self.assertEqual(x, Array("i", [1, 2, 10, 11, 12]))
        x = Array("i", (1, 2, 3))
        x[1:-1] = Array("i", [10, 11, 12])
        self.assertEqual(x, Array("i", [1, 10, 11, 12, 3]))
        del x[:2]
        self.assertEqual(x, Array("i", [11, 12, 3]))
        
    def test_count(self):
        x = Array("i", (1, 2, 3, 2))
        self.assertEqual(x.count(2), 2)
        self.assertEqual(x.count(3), 1)
        self.assertEqual(x.count(4), 0)

    def test_iter(self):
        x = Array("i", (1, 2, 3, 2))
        self.assert_(2 in x)
        self.assert_(5 not in x)
        g = iter(x)
        self.assertEqual(list(g), [1, 2, 3, 2])

    def test_index(self):
        x = Array("i", (1, 2, 3, 2))
        self.assertEqual(x.index(2), 1)
        self.assertEqual(x.rindex(2), 3)
        self.assertEqual(x.index(2,2), 3)
        self.assertRaises(ValueError, x.index, 10)

    def test_len(self):
        self.assertEqual(len(Array("i",(2,2))), 2)
        self.assertEqual(len(Array("f")), 0)

    def test_cmp(self):
        self.assert_(Array("i", [1, 2, 3]) < [2, 3])
        self.assert_(Array("i", [1, 2, 3]) > Array("i", [0, 2, 3]))
        self.assert_(Array("i", [1, 2, 3]) == Array("f", [1.0, 2, 3]))

    def test_mul(self):
        x = Array("i", (1, 2, 3, 2))
        x *= 2
        # __mul__ only for Array subclass
        self.assertEqual(Array("i", [1, 2])*2, Array("i", [1, 2, 1, 2]))

    def test_add(self):
        x = Array("i", (1, 2, 3, 2))
        x += Array("i", (1, 2))
        self.assertEqual(x , Array("i", [1, 2, 3, 2, 1, 2]))
        x.extend([10,11])
        self.assertEqual(x , Array("i", [1, 2, 3, 2, 1, 2, 10, 11])) 
        # __add__ only defined for Array subclass
        self.assertEqual(Array("i", [1, 2])+ Array("i", [10, 11]),
                         Array("i", [1, 2, 10, 11]))

    def test_insertpopremove(self):
        x = Array("i", (1, 2))
        x.insert(1,10)
        self.assertEqual(x , Array("i", [1, 10, 2]))
        self.assertEqual(x.pop(0) ,1)
        self.assertEqual(x , Array("i", [10, 2]))
        x.remove(10)
        self.assertEqual(x , Array("i", [2]))
        x.remove(10)
        self.assertEqual(x , Array("i", [2]))

    def test_reversesort(self):
        x = Array("i", (3, 1, 2))
        x.reverse()
        self.assertEqual(x , Array("i", [2, 1, 3]))
        x.sort()
        self.assertEqual(x , Array("i", [1, 2, 3]))

        
from test.test_support import run_suite
def test_main():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(StrArrayTester))
    run_suite(suite)

if __name__ == '__main__':
    test_main()
