"Test chunk object"


import unittest
from mem.chunk import chunk

import pickle

class ChunkTester(unittest.TestCase):

    def test_create(self):
        x = chunk()
        self.assertEqual(str(x), "")
        x = chunk("*-*")
        self.assertEqual(str(x), "*-*")
        x = chunk("a", "bc", "d", "ef")
        self.assertEqual(str(x), "abcdef")
        
        c = chunk("*")
        self.assertEqual(str(c), "*")
        c.resize(3)
        self.assertEqual(str(c), "*\0\0")
        c.resize(1)
        self.assertEqual(str(c), "*")
        c.resize(4, "+-")
        self.assertEqual(str(c), "*+-+")
        self.assertRaises(ValueError, c.resize, 20, "")

    def test_add(self):
        a = chunk("*")
        self.assertEqual(str(a+a), "**")
        self.assertEqual(str(a+"+"), "*+")
        b = a
        a += "+"
        self.assertEqual(str(b), "*+")

    def test_mul(self):
        a = chunk("*")
        self.assertEqual(str(a*2), "**")
        self.assertEqual(str(a*1), "*")
        self.assertEqual(str(a*0), "")
        self.assertEqual(str(a*-1), "")
        b = a
        a *= 2
        self.assertEqual(str(b), "**")

    def test_getsetitem(self):
        a = chunk("abc")
        self.assertEqual(a[0], "a")
        self.assertEqual(a[-1], "c")
        self.assertRaises(IndexError, a.__getitem__, -10)
        self.assertRaises(IndexError, a.__getitem__, 4)
        a[1] = "*"
        self.assertEqual(str(a), "a*c")
        self.assertRaises(ValueError, a.__setitem__, 1, "??")
        for i,v in [(1, "ac"), (0, "c"), (0, "")]:
            del a[i]
            self.assertEqual(str(a), v)

    def test_getsetslice(self):

        a = chunk("abc")
        self.assertEqual(str(a[1:]), "bc")
        self.assertEqual(str(a[:-1]), "ab")
        a[1:2] = "*"
        self.assertEqual(str(a), "a*c")
        a[1:2] = ""
        self.assertEqual(str(a), "ac")
        a[1:1] = a
        self.assertEqual(str(a), "aacc")
        del a[1:-1]
        self.assertEqual(str(a), "ac")
        del a[:]
        self.assertEqual(str(a), "")

        # test for slicing by copy behavior
        a = chunk("1234")
        b = a[1:]
        b[0] = "0"
        self.assertEqual(str(b), "034")
        self.assertEqual(str(a), "1234")

        #check for a bug in version 0.1
        a = chunk("123554")
        self.assertEqual(str(a[-3:-8]), "")
        

    def test_contains(self):
        a = chunk("abc")
        self.assert_("a" in a)
        self.assert_("bc" in a)
        self.assert_("d" not in a)
        self.assert_("ac" not in a)
        self.assert_(""  in a)

    def test_compare(self):
        a = chunk("abc")
        b = chunk("bc")
        c = chunk("bc")
        d = chunk("cd")
        self.assert_(a<b)
        self.assert_(c==b)
        self.assert_(d>b)
        self.assert_(a!=d)


    def test_subclass(self):
        class A(chunk):
            def __new__(cls, *args):
                obj = chunk.__new__(cls, *args)
                obj.args = args
                return obj
        a = A("****")
        self.assertEqual(str(a), "****")
        self.assertEqual(a.args, ("****",))

    def test_pickle(self):
        a = chunk("Hello world")
        s = pickle.dumps(a)
        b = pickle.loads(s)
        self.assertEqual(type(a), type(b))
        self.assertEqual(a, b)
        

from test.test_support import run_suite
def test_main():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ChunkTester))
    run_suite(suite)

if __name__ == '__main__':
    test_main()
