"Test mem.tools module"

import unittest
from mem.chunk import chunk
from mem.tools import memcopy, memcmp, memlen, index, rindex
from mem.tools import match, diff, rmatch, rdiff

class MemToolsTester(unittest.TestCase):

    def test_memcopy(self):
        x = chunk("*"*5)
        memcopy(x, "+++", 1)
        self.assertEqual(str(x), "*+++*")
        memcopy(x, "oo")
        self.assertEqual(str(x), "oo++*")
        self.assertRaises(IndexError, memcopy, x, "123456789")

    def test_index(self):
        s = " 12 12 12 "
        self.assertEqual(index(s,"12"), 1)
        self.assertEqual(index(s,"12",0,len(s)), 1)
        self.assertEqual(index(s,"12",0,len(s),2), 4)
        self.assertEqual(index(s,"12",2,len(s)), 4)
        self.assertRaises(ValueError, index, s, "12", 0, 2)
        self.assertEqual(rindex(s,"12"), 7)
        self.assertEqual(rindex(s,"12",0,len(s)), 7)
        self.assertEqual(rindex(s,"12",0,len(s),2), 4)
        self.assertEqual(rindex(s,"12",0,len(s)-2), 4)
        self.assertRaises(ValueError, rindex, s, "12", 0, 2)

    def test_memcmp(self):
        self.assertEqual(memcmp(chunk(3*"*"), "***"), 0)
        self.assertEqual(memcmp("a", "b"), -1)
        self.assertEqual(memcmp("ac", "a"), 1)

    def test_memlen(self):
        self.assertEqual(memlen(""), 0)
        self.assertEqual(memlen("1234"), 4)

    def test_match(self):
        self.assertEqual(match("", ""), -1)
        self.assertEqual(match("123","*23"), 1)
        self.assertEqual(match("123","12*"), 0)
        self.assertEqual(match("123","231"), -1)
        self.assertEqual(match("123","*234"), 1)

    def test_rmatch(self):
        self.assertEqual(rmatch("", ""), -1)
        self.assertEqual(rmatch("123","*23"), 2)
        self.assertEqual(rmatch("123","12*"), 1)
        self.assertEqual(rmatch("123","231"), -1)
        self.assertEqual(rmatch("23*","1234"), 1)

    def test_diff(self):
        self.assertEqual(diff("", ""), -1)
        self.assertEqual(diff("123","*23"), 0)
        self.assertEqual(diff("123","12*"), 2)
        self.assertEqual(diff("123","123"), -1)
        self.assertEqual(diff("12*","1234"), 2)
        
    def test_rdiff(self):
        self.assertEqual(rdiff("", ""), -1)
        self.assertEqual(rdiff("123","*23"), 0)
        self.assertEqual(rdiff("123","12*"), 2)
        self.assertEqual(rdiff("123","123"), -1)
        self.assertEqual(rdiff("*34","1234"), 0)

from test.test_support import run_suite
def test_main():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(MemToolsTester))
    run_suite(suite)

if __name__ == '__main__':
    test_main()
